# Resource

from culqipy.resource import (
    Token,
    Charge,
    Plan,
    Subscription,
    Refund,
    Iins,
    Card,
    Event,
    Customer,
    Transfer
)

# Configuration variables

API_URL = "https://api.culqi.com/v2"
public_key = None
secret_key = None
